/*	Cory McWilliams
		mcwilc@rpi.edu
		10/30/2003 */

#include "objects.h"
#include <GL/glut.h>
#include <math.h>
#include <iostream>

// Some sensible constants
const float black[4] = {0.0f, 0.0f, 0.0f, 0.0f};
const float white[4] = {1.0f, 1.0f, 1.0f, 0.0f};
const float bark[4] = {0.4f, 0.2f, 0.0f, 0.0f};
const float leaves[4] = {0.0f, 0.4f, 0.0f, 0.0f};
GLUquadric *gluq;
unsigned int texLeaves;
unsigned int texBark;

// Load textures, initialize state
void initObjects() {
	gluq = gluNewQuadric();
	gluQuadricTexture(gluq, GL_TRUE);

	glEnable(GL_TEXTURE_2D);
	glGenTextures(1, &texLeaves);
	glGenTextures(1, &texBark);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

	FILE *f;
	size_t size = 256*256;
	if ((f = fopen("leaves.rgb", "rb"))) {
		char buffer[size*3];
		if (fread(buffer, 3, size, f) == size) {
			glBindTexture(GL_TEXTURE_2D, texLeaves);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 256, 256, 0, GL_RGB, GL_UNSIGNED_BYTE, buffer);
		} else
			std::cerr << "Error reading leaves.rgb\n";
		fclose(f);
	} else
		std::cerr << "Error loading leaves.rgb\n";
	if ((f = fopen("bark.rgb", "rb"))) {
		char buffer[size*3];
		if (fread(buffer, 3, size, f) == size) {
			glBindTexture(GL_TEXTURE_2D, texBark);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 256, 256, 0, GL_RGB, GL_UNSIGNED_BYTE, buffer);
		} else
			std::cerr << "Error reading bark.rgb\n";
		fclose(f);
	} else
		std::cerr << "Error loading bark.rgb\n";
}

// Draw a maple tree, this time
void tree(float x, float z) {
	glPushMatrix();
	glTranslatef(x, 0.0f, z);
	glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);
	treeFractal(2, 0.4f, 2.0f);
	glPopMatrix();
}

// The trees are fractal trees, as best as I felt like managing on the fly
void treeFractal(int levels, float radius, float height) {
	const float scaleDown = 0.9f;
	const int branches = 3;
	if (levels < 0) { // end condition
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, leaves);
		glBindTexture(GL_TEXTURE_2D, texLeaves);
		gluSphere(gluq, height*0.85f, 5, 5);
		glBindTexture(GL_TEXTURE_2D, 0);
		return;
	}
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, bark);
	glBindTexture(GL_TEXTURE_2D, texBark);
	gluCylinder(gluq, radius, scaleDown * radius, height, 5, 1);
	glBindTexture(GL_TEXTURE_2D, 0);
	for (int i = 0; i < branches; i++) {
		glPushMatrix();
		glTranslatef(0.0f, 0.0f, height);
		glRotatef(i * 360.0f / branches, 0.0f, 0.0f, 1.0f);
		glRotatef(30.0f, 0.0f, 1.0f, 0.0f);
		treeFractal(levels - 1, radius * scaleDown, height * scaleDown); // recurse
		glPopMatrix();
		if (levels - 1 < 0) return;
	}
}

// Draw a bird using bezier surfaces
void bird(float f) {
	// Curve control points
	float controlPoints[4][4][3] = {
		{{0.0f, 0.0f, 1.0f}, {0.0f, 0.0f, 0.66f}, {0.0f, 0.0f, 0.33f}, {0.0f, 0.0f, 0.0f}},
		{{0.33f, f*0.5f + 0.1f, 1.0f}, {0.33f, f*0.5f + 0.1f, 0.66f}, {0.33f, f*0.5f + 0.1f, 0.33f}, {0.33f, f*0.5f + 0.1f, 0.0f}},
		{{0.66f, f*0.2f, 1.0f}, {0.66f, f*0.2f, 0.66f}, {0.66f, f*0.2f, 0.33f}, {0.66f, f*0.2f, 0.0f}},
		{{1.0f, f*-0.5f, 0.0f}, {1.0f, f*-0.5f, 0.0f}, {1.0f, f*-0.5f, 0.0f}, {1.0f, f*-0.5f, 0.0f}},
	};

	glMap2f(GL_MAP2_VERTEX_3, 0, 1, 3, 4, 0, 1, 12, 4, &controlPoints[0][0][0]);
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, black);

	glPushMatrix();
	int isteps = 5;
	float steps = float(isteps);
	glBegin(GL_QUADS);
	for (int i = 0; i < isteps; i++) { // A nice curve
		for (int j = 0; j < isteps; j++) {
			glEvalCoord2f(j / steps, i / steps);
			glEvalCoord2f((j+1) / steps, i / steps);
			glEvalCoord2f((j+1) / steps, (i+1) / steps);
			glEvalCoord2f(j / steps, (i+1) / steps);
		}
	}
	glEnd();
	glScalef(-1.0f, 1.0f, 1.0f);
	glBegin(GL_QUADS);
	for (int i = 0; i < isteps; i++) { // The same curve, but on the other side
		for (int j = 0; j < isteps; j++) {
			glEvalCoord2f(j / steps, i / steps);
			glEvalCoord2f((j+1) / steps, i / steps);
			glEvalCoord2f((j+1) / steps, (i+1) / steps);
			glEvalCoord2f(j / steps, (i+1) / steps);
		}
	}
	glEnd();
	glPopMatrix();
}
