/*	Cory McWilliams
		mcwilc@rpi.edu
		10/30/2003 */

#include "squirrel.h"
#include <iostream>

// Constructor...just create a quadratic and initialize variables
Squirrel::Squirrel() {
	if (!(gluq = gluNewQuadric())) {
		std::cerr << "Unable to create GLU quadric object.\n";
		exit(1);
	}
	for (int i = 0; i < PART_COUNT; rot[i++] = 0.0f);
}

// Draw the great, marvelous squirrel
void Squirrel::draw() {
	float fur[4] = {0.3f, 0.1f, 0.0f, 1.0f};
	float black[4] = {0.0f, 0.0f, 0.0f, 0.0f};
	const int slices = 16;
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, fur);
	glPushMatrix();
	glRotatef(-90.0f, 1.0f, 0.0f, 0.0f);
	glTranslatef(0.0f, 0.0f, 1.2f);
	gluSphere(gluq, 0.5f, slices, slices); // Lower body

		glPushMatrix();
		glTranslatef(0.3536f, 0.0f, -0.35361f);
		glRotatef(180.0f, 1.0f, 0.0f, 0.0f);
		glRotatef(rot[PART_LEGS_OUT], 0.0f, 1.0f, 0.0f);
		glRotatef(rot[PART_RLEG_FORWARD], 1.0f, 0.0f, 0.0f);
		gluSphere(gluq, 0.1f, slices, slices); // Right hip
		gluCylinder(gluq, 0.1f, 0.1f, 0.3f, slices, 2); // Right leg
		glTranslatef(0.0f, 0.0f, 0.3f);
		glRotatef(-rot[PART_RLEG_FLEX], 1.0f, 0.0f, 0.0f);
		gluSphere(gluq, 0.1f, slices, slices); // Right knee
		gluCylinder(gluq, 0.1f, 0.1f, 0.4f, slices, 2); // Right shin
		glTranslatef(0.0f, 0.0f, 0.4f);
		gluSphere(gluq, 0.2f, slices, slices); // Right foot
		glPopMatrix();

		glPushMatrix();
		glTranslatef(-0.3536f, 0.0f, -0.35361f);
		glRotatef(180.0f, 1.0f, 0.0f, 0.0f);
		glRotatef(-rot[PART_LEGS_OUT], 0.0f, 1.0f, 0.0f);
		glRotatef(rot[PART_LLEG_FORWARD], 1.0f, 0.0f, 0.0f);
		gluSphere(gluq, 0.1f, slices, slices); // Left hip
		gluCylinder(gluq, 0.1f, 0.1f, 0.3f, slices, 2); // Left leg
		glTranslatef(0.0f, 0.0f, 0.3f);
		glRotatef(-rot[PART_LLEG_FLEX], 1.0f, 0.0f, 0.0f);
		gluSphere(gluq, 0.1f, slices, slices); // Left knee
		gluCylinder(gluq, 0.1f, 0.1f, 0.4f, slices, 2); // Left shin
		glTranslatef(0.0f, 0.0f, 0.4f);
		gluSphere(gluq, 0.2f, slices, slices); // Left foot
		glPopMatrix();

		glPushMatrix();
		glTranslatef(0.0f, -0.3536f, -0.3536f);
		glRotatef(rot[PART_TAIL0] + 90.0f, 1.0f, 0.0f, 0.0f);
		glRotatef(rot[PART_TAILY], 0.0f, 1.0f, 0.0f);
		gluSphere(gluq, 0.1f, slices, slices); // Tail O 1
		gluCylinder(gluq, 0.1f, 0.3f, 0.7f, slices, 2); // Tail = 1
		glTranslatef(0.0f, 0.0f, 0.7f);
		glRotatef(rot[PART_TAIL0], 1.0f, 0.0f, 0.0f);
		glRotatef(rot[PART_TAILY], 0.0f, 1.0f, 0.0f);
		gluSphere(gluq, 0.3f, slices, slices); // Tail O 1
		gluCylinder(gluq, 0.3f, 0.4f, 0.7f, slices, 2); // Tail = 2
		glTranslatef(0.0f, 0.0f, 0.7f);
		glRotatef(rot[PART_TAIL0], 1.0f, 0.0f, 0.0f);
		glRotatef(rot[PART_TAILY], 0.0f, 1.0f, 0.0f);
		gluSphere(gluq, 0.4f, slices, slices); // Tail O 2
		gluCylinder(gluq, 0.4f, 0.4f, 1.0f, slices, 2); // Tail = 3
		glTranslatef(0.0f, 0.0f, 1.0f);
		glRotatef(rot[PART_TAIL1], 1.0f, 0.0f, 0.0f);
		glRotatef(rot[PART_TAILY], 0.0f, 1.0f, 0.0f);
		gluSphere(gluq, 0.4f, slices, slices); // Tail O 3
		gluCylinder(gluq, 0.4f, 0.3f, 0.7f, slices, 2); // Tail = 4
		glTranslatef(0.0f, 0.0f, 0.7f);
		gluSphere(gluq, 0.3f, slices, slices); // Tail O 4
		glPopMatrix();

	gluCylinder(gluq, 0.5f, 0.5f, 1.0f, slices, 2); // Torso

		glPushMatrix();
		glTranslatef(0.5f, 0.0f, 0.5f);
		glRotatef(90.0f, 0.0f, 1.0f, 0.0f); 
		glRotatef(-rot[PART_ARMS_IN], 1.0f, 0.0f, 0.0f);
		glRotatef(90.0f-rot[PART_RARM_FORWARD], 0.0f, 1.0f, 0.0f);
		gluSphere(gluq, 0.1f, slices, slices); // Right shoulder
		gluCylinder(gluq, 0.1f, 0.1f, 0.3f, slices, 2); // Right arm
		glTranslatef(0.0f, 0.0f, 0.3f);
		glRotatef(-rot[PART_RARM_FLEX], 0.0f, 1.0f, 0.0f);
		gluSphere(gluq, 0.1f, slices, slices); // Right elbow
		gluCylinder(gluq, 0.1f, 0.1f, 0.4f, slices, 2); // Right wrist
		glTranslatef(0.0f, 0.0f, 0.4f);
		gluSphere(gluq, 0.2f, slices, slices); // Right hand
		glPopMatrix();

		glPushMatrix();
		glTranslatef(-0.5f, 0.0f, 0.5f);
		glRotatef(-90.0f, 0.0f, 1.0f, 0.0f);
		glRotatef(-rot[PART_ARMS_IN], 1.0f, 0.0f, 0.0f);
		glRotatef(-90.0f+rot[PART_LARM_FORWARD], 0.0f, 1.0f, 0.0f);
		gluSphere(gluq, 0.1f, slices, slices); // Left shoulder
		gluCylinder(gluq, 0.1f, 0.1f, 0.3f, slices, 2); // Left arm
		glTranslatef(0.0f, 0.0f, 0.3f);
		glRotatef(rot[PART_LARM_FLEX], 0.0f, 1.0f, 0.0f);
		gluSphere(gluq, 0.1f, slices, slices); // Left elbow
		gluCylinder(gluq, 0.1f, 0.1f, 0.4f, slices, 2); // Left wrist
		glTranslatef(0.0f, 0.0f, 0.4f);
		gluSphere(gluq, 0.2f, slices, slices); // Left hand
		glPopMatrix();

	glTranslatef(0.0f, 0.0f, 1.0f);
	//gluSphere(gluq, 0.5f, slices, slices); // Top of head
	gluDisk(gluq, 0.0f, 0.5f, slices, 1);
		
		glPushMatrix();
		glTranslatef(0.3f, 0.0f, 0.0f);
		gluSphere(gluq, 0.2f, slices, slices); // Right ear 
		glPopMatrix();

		glPushMatrix();
		glTranslatef(-0.3f, 0.0f, 0.0f);
		gluSphere(gluq, 0.2f, slices, slices); // Left ear
		glPopMatrix();
		
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, black);
	glTranslatef(0.0f, 0.0f, -0.3f);

		glPushMatrix();
		glTranslatef(0.1f, 0.4f, 0.0f);
		gluSphere(gluq, 0.1f, slices, slices); // Left elbow
		glPopMatrix();

		glPushMatrix();
		glTranslatef(-0.1f, 0.4f, 0.0f);
		gluSphere(gluq, 0.1f, slices, slices); // Left elbow
		glPopMatrix();

	glPopMatrix();
}
