/*	Cory McWilliams
		mcwilc@rpi.edu
		10/30/2003 */

#if defined (__APPLE__)
#include <GLUT/glut.h>
#include <libgen.h>
#else
#include <GL/glut.h>
#endif
#include <cmath>
#include <iostream>
#include "squirrel.h"
#include "objects.h"

// A few macros to save me typing later on
#define ABS(a) ((a)>0?(a):(-(a)))
#define ANIMATE(a,i,n,p,t) squirrel.part(p)=catmullRomSpline(a+i[n], i[n+1]-i[n], t)

// Global state variables
float position[3] = {0.0f, 2.75f, 5.0f};
float rot[3];
bool buttons[3];
int ww;
int wh;
int lastx;
int lasty;
bool keys[256];
enum Action { Standing, StartRun, Running, StopRun } action = Standing;
Squirrel squirrel;
bool arrows[4];
float spos[3] = {0.0f, 0.0f, 0.0f};
float srot[3] = {0.0f, 0.0f, 0.0f};
float bpos[3] = {0.0f, 8.0f, 0.0f};
float brot = 0.0f;
float flap = 0.0f;

// This defines the squirrel's animation
float animation[] = {
	// Standing
	-30.0f, -30.0f, -30.0f, -30.0f, -30.0f, 0.0f, -30.0f, 0.0f,
	50.0f, -10.0f, 50.0f, -20.0f,
	0.0f,
	0.0f,
	0.0f,
	0.0f,

	// Running
	-15.0f, 0.0f, -12.0f, 0.0f,
	12.0f, 0.0f, 15.0f, 0.0f,
	30.0f, 86.0f,
	30.0f, 20.0f,
	80.0f, -40.0f,
	30.0f, 50.0f
};

// Some nice places to put trees
float trees[][2] = {
	{3.0f, 0.0f},
	{10.0f, 2.0f},
	{34.0f, 6.0f},
	{32.0f, 1.0f},
	{9.0f, 30.0f},
	{3.0f, 20.0f},
	{2.0f, 10.0f},
	{30.0f, 24.0f},
	{15.0f, 15.0f},
	{12.0f, 10.0f},
	{18.0f, 26.0f}
};

// Indexes into the numbers in the animation array
int standIndex[7] = {0, 8, 12, 13, 14, 15, 16};
int runIndex[7] = {16, 20, 24, 26, 28, 30, 32};

// Calculating curves is fun
float catmullRom(float v1, float v2, float v3, float v4, float t) {
	return 0.5f * ((2 * v2) + (-1.0f * v1 + v3) * t +
		(2.0f * v1 - 5.0f * v2 + 4.0f * v3 - v4) * t * t +
		(-1.0f * v1 + 3.0f * v2 - 3.0f * v3 + v4) * t * t * t);
}

// Calculating splines from curves is even more fun
float catmullRomSpline(float *v, int count, float t) {
	int segment = int(t * count);
	t = (t * count - segment);
	return catmullRom(v[segment % count], v[(segment + 1) % count],
		v[(segment + 2) % count], v[(segment + 3) % count], t);
}

// Some simple initialization
void init() {
	glClearColor(0.0f, 0.0f, 1.0f, 1.0f);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_NORMALIZE);
	glShadeModel(GL_SMOOTH);
	glEnable(GL_MAP2_VERTEX_3);

	// Lighting
	float white[4] = {1.0f, 1.0f, 1.0f, 0.0f};
	float ambient[4] = {0.3f, 0.3f, 0.3f, 0.0f};
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, white);
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);

	// Initialize the squirrel's pose
	squirrel.part(PART_ARMS_IN) = 60.0f;

	initObjects();
}

// All drawing is done within the scope of this function
void display() {
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	glPushMatrix();

	// Camera position
	glLoadIdentity();
	gluLookAt(position[0], position[1], position[2], spos[0], position[1], spos[2], 0.0f, 1.0f, 0.0f);

	// Light position
	float pos[4] = {10.0f, 10.0f, 10.0f, 0.0f};
	glLightfv(GL_LIGHT0, GL_POSITION, pos);

	// The ground
	float grass[4] = {0.0f, 0.4f, 0.0f, 0.0f};
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, grass); 
	glNormal3f(0.0f, 1.0f, 0.0f);
	glBegin(GL_QUADS);
	glVertex3f(-40.0f, 0.0f, -40.0f);
	glVertex3f(40.0f, 0.0f, -40.0f);
	glVertex3f(40.0f, 0.0f, 100.0f);
	glVertex3f(-40.0f, 0.0f, 100.0f);
	glEnd();

	// Trees
	for (size_t i = 0; i < sizeof(trees) / sizeof(*trees); i++)
		tree(trees[i][0], trees[i][1]);
	
	// A bird
	glPushMatrix();
	glTranslatef(bpos[0], bpos[1], bpos[2]);
	glRotatef(brot, 0.0f, 1.0f, 0.0f);
	bird(ABS(flap));
	glPopMatrix();

	// The squirrel
	glTranslatef(spos[0], spos[1], spos[2]);
	glRotatef(srot[0], 1.0f, 0.0f, 0.0f);
	glRotatef(srot[1], 0.0f, 1.0f, 0.0f);
	glRotatef(srot[2], 0.0f, 0.0f, 1.0f);
	squirrel.draw();

	glPopMatrix();
	glutSwapBuffers();
}

// Simple perspective
void reshape(int width, int height) {
	ww = width;
	wh = height;
	if (!height) height=1; // prevent division by zero
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.0f, float(width)/float(height), 0.01f, 100.0f);
	glMatrixMode(GL_MODELVIEW);
}

void keyboard(unsigned char key, int x, int y) {
	keys[key] = true;
}

// Update key table
void keyboardUp(unsigned char key, int x, int y) {
	keys[key] = false;
}

// Maintain the state of the arrow keys
void special(int key, int x, int y) {
	switch (key) {
	case GLUT_KEY_UP: arrows[0] = true; break;
	case GLUT_KEY_RIGHT: arrows[1] = true; break;
	case GLUT_KEY_DOWN: arrows[2] = true; break;
	case GLUT_KEY_LEFT: arrows[3] = true; break;
	}
}

// Maintain the state of the arrow keys
void specialUp(int key, int x, int y) {
	switch (key) {
	case GLUT_KEY_UP: arrows[0] = false; break;
	case GLUT_KEY_RIGHT: arrows[1] = false; break;
	case GLUT_KEY_DOWN: arrows[2] = false; break;
	case GLUT_KEY_LEFT: arrows[3] = false; break;
	}
}

// Lots of animation goodness
void timer(int t) {
  // Move the squirrel...but it's not that easy, because it is necessary to take angle into account
	float origPos[3] = {spos[0], spos[1], spos[2]};
  float step = 0.100f;
	float theta = float(-srot[1] * M_PI / 180.0f);
  float zstep = cos(theta) * step;
  float xstep = sin(theta) * step;
  if (arrows[0]) { spos[0] += xstep; spos[2] -= zstep; }
  if (arrows[2]) { spos[0] -= xstep; spos[2] += zstep; }
  if (arrows[1]) srot[1] -= 45.0f * step;
  if (arrows[3]) srot[1] += 45.0f * step;

	// Move the bird
	theta = float(-brot * M_PI / 180.0f);
	zstep = cos(theta) * step;
	xstep = sin(theta) * step;
	bpos[0] += xstep;
	bpos[2] -= zstep;

	// Make the bird's wings 'flap'
	if (flap >= 0.0f) {
		flap += 0.05f;
		if (flap > 1.0f) flap = -1.0f;
	} else {
		flap += 0.05f;
	}

	// Keep the bird within a circle
	float birdBound = 30.0f;
	float dist2 = bpos[0] * bpos[0] + bpos[2] * bpos[2];
	if (dist2 > birdBound * birdBound)
		brot += 3.0f;
	else
		brot -= 0.1f;
	if (brot > 360.0f) brot -= 360.0f;
	else if (brot < 0.0f) brot += 360.0f;

	// Make the tail move around in a cool way
	if (arrows[1] && squirrel.part(PART_TAILY) < 0.0f || arrows[3] && squirrel.part(PART_TAILY) > 0.0f)
		squirrel.part(PART_TAILY) *= 0.85f;
	else
		squirrel.part(PART_TAILY) *= 0.9f;
	if (arrows[1]) squirrel.part(PART_TAILY) += 3.0f;
	if (arrows[3]) squirrel.part(PART_TAILY) -= 3.0f;

	// Respond to keyboard input by changing animations for the squirrel
	if ((arrows[0] || arrows[2]) && action == Standing) action = StartRun;
	if (!arrows[0] && !arrows[2] && (action == Running)) action = StopRun;

	// All of these variables are preparation to do nice, smooth animations with the squirrel
	static float midPoints[PART_COUNT][4];
	int midIndex[] = {0, 2, 4, 6, 8, 10, 12};
	
	static float val = 0.0f;
	static float increment = 0.02f;
	static Action lastAction = action;
	int *base = runIndex;

	val += increment;
	if (val > 1.0f) val -= 1.0f;
	float other = val + 0.5f;
	if (other > 1.0f) other -= 1.0f;

	static float inc[4] = {0.0006f, 0.02f};
	bool draw = false;
	
	switch (action) {
	case Standing:	// Standing and running are the _simple_ cases here
		increment = inc[0];
		base = standIndex;
		draw = true;
		break;
	case Running:
		increment = inc[1];
		draw = true;
		break;
	case StopRun:
	case StartRun:	// Transitioning between standing and running was much more difficult than I hoped
		base = midIndex;
		if (lastAction == Standing || lastAction == Running) { // The first time, interpolate the poses to transition between
			for (int i = 0; i < PART_COUNT; i++) {
				if (i == PART_ARMS_IN || i == PART_LEGS_OUT || i == PART_TAILY) continue;
				bool o = (i==PART_RARM_FORWARD||i==PART_RARM_FLEX||i==PART_RLEG_FORWARD||i==PART_RLEG_FLEX);
				int map[] = {0, 1, -1, 2, 2, 3, 3, -1, 4, 4, 5, 5};
				int m = map[i];
				int *a = (action == StartRun)?standIndex:runIndex;
				int *b = (action != StartRun)?standIndex:runIndex;
				midPoints[i][0] = catmullRomSpline(animation+a[m], a[m+1]-a[m], (o?other:val)-inc[0]);
				midPoints[i][1] = catmullRomSpline(animation+a[m], a[m+1]-a[m], o?other:val);
				midPoints[i][2] = catmullRomSpline(animation+b[m], b[m+1]-b[m], o?0.5f:0.0f);
				midPoints[i][3] = catmullRomSpline(animation+b[m], b[m+1]-b[m], (o?0.5f:0.0f)+inc[1]);
			}
			val = 0.0f;
		}
		for (int i = 0; i < PART_COUNT; i++)	// Interpolate the pose between the two end poses
			if (i != PART_ARMS_IN && i != PART_LEGS_OUT && i != PART_TAILY)
				squirrel.part(i) = 
					catmullRom(midPoints[i][0], midPoints[i][1], midPoints[i][2], midPoints[i][3], val);

		increment = 0.1f;

		if (val + increment > 1.0f) { // If the transition completed, go back to standing/running
			action = (action == StartRun)?Running:Standing;
			val = 0.0f;
		}
		break;
	}

	lastAction = action;

	if (draw) { // Set each joint of the squirrel to its calculated value
		ANIMATE(animation, base, 0, PART_TAIL0, val);
		ANIMATE(animation, base, 1, PART_TAIL1, val);
		ANIMATE(animation, base, 2, PART_LARM_FORWARD, val);
		ANIMATE(animation, base, 2, PART_RARM_FORWARD, other);
		ANIMATE(animation, base, 3, PART_LARM_FLEX, val);
		ANIMATE(animation, base, 3, PART_RARM_FLEX, other);
		ANIMATE(animation, base, 4, PART_LLEG_FORWARD, val);
		ANIMATE(animation, base, 4, PART_RLEG_FORWARD, other);
		ANIMATE(animation, base, 5, PART_LLEG_FLEX, val);
		ANIMATE(animation, base, 5, PART_RLEG_FLEX, other);
	}
	
	// Keep our friends outside of the trees
	for (size_t i = 0; i < sizeof(trees) / sizeof(*trees); i++) {
		float diff[2] = {spos[0] - trees[i][0], spos[2] - trees[i][1]};
		float mag2 = diff[0] * diff[0] + diff[1] * diff[1];
		float min = 1.0f;
		if (mag2 < min * min && mag2 >= 0.1f)
			for (int j = 0; j < 3; j++) spos[j] = origPos[j];
	}

	// Relate the squirrel position and camera position by keeping them between given bounds.
	float min = 9.0f;
	float max = 10.0f;
	float diff[3] = {position[0] - spos[0], position[1] - spos[1], position[2] - spos[2]};
	float mag2 = diff[0] * diff[0] + diff[1] * diff[1] + diff[2] * diff[2];
	if (mag2 > max * max) {
		for (int i = 0; i < 3; i++) if (i != 1) position[i] = position[i] + (1.0f - max / sqrt(mag2)) * -diff[i];
	} else if (mag2 < min * min) {
		for (int i = 0; i < 3; i++) if (i != 1) position[i] = position[i] + (1.0f - min / sqrt(mag2)) * -diff[i];
	}

	// Set another timer and redraw
	glutPostRedisplay();
	glutTimerFunc(10, timer, 0);
}

// Program entry point
int main(int argc, char *argv[]) {
#ifdef __APPLE__
	if (argc >= 1 && chdir(dirname(argv[0])) == 0)
	{
		chdir("../Resources");
	}
#endif
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB|GLUT_DOUBLE|GLUT_DEPTH);
	glutInitWindowSize(640, 480);
	glutCreateWindow(argv[0]);
	init();
	glutIgnoreKeyRepeat(true);
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutTimerFunc(0, timer, 0);
	glutKeyboardFunc(keyboard);
	glutKeyboardUpFunc(keyboardUp);
	glutSpecialFunc(special);
	glutSpecialUpFunc(specialUp);
	glutMainLoop();
	return 0;
}
